/* 
 * Get the MAC address of interfaces given in command line
 * 
 * Written by Pozsar Balazs <pozsy@sch.bme.hu>
 * based on nameif.c written 2000 by Andi Kleen.
 * Subject to the Gnu Public License, version 2.  
 */

#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <linux/sockios.h>

int ctl_sk = -1;

void warning(char *fmt, ...)
{
    va_list ap;
    va_start(ap, fmt);
    vfprintf(stderr, fmt, ap);
    va_end(ap);
    fputc('\n', stderr);
}

void opensock(void)
{
    if (ctl_sk < 0)
	ctl_sk = socket(PF_INET, SOCK_DGRAM, 0);
}

int getmac(char *name, unsigned char *mac)
{
    int r;
    struct ifreq ifr;
    opensock();
    memset(&ifr, 0, sizeof(struct ifreq));
    strcpy(ifr.ifr_name, name);
    r = ioctl(ctl_sk, SIOCGIFHWADDR, &ifr);
    memcpy(mac, ifr.ifr_hwaddr.sa_data, 6);
    return r;
}

int main(int argc, char **argv)
{
    unsigned char mac[6];

    while (++argv, --argc) {
	if (strlen(argv[0]) > IFNAMSIZ - 1) {
	    warning("interface name %s too long", argv[0]);
	    continue;
	}

	if (getmac(argv[0], mac) < 0) {
	    warning("error while reading mac of %s", argv[0]);
	    continue;
	}

	printf("%02x:%02x:%02x:%02x:%02x:%02x\n", mac[0], mac[1],
	       mac[2], mac[3], mac[4], mac[5]);
    }

    return 0;
}
